local version = "1.3";
EnhancedAssistOptions = {};
local myClass = UnitClass("player");
local myName = UnitName("player");

local function InitializeSetup()
	-- The following will upgrade old binary settings to the new english settings. This code can be removed after version 1.0
	--[[for index, setting in EnhancedAssistOptions do
		if ( EnhancedAssistOptions[index] == "on" ) then
			EnhancedAssistOptions[index] = 1;
		elseif ( EnhancedAssistOptions[index] == "off" ) then
			EnhancedAssistOptions[index] = 0;
		end
	end]]

	-- Set defaults if none are set yet
	if(EnhancedAssistOptions == nil)						then EnhancedAssistOptions = {} end;
	if(EnhancedAssistOptions.Show == nil)					then EnhancedAssistOptions.Show = 1 end;
	if(EnhancedAssistOptions.CurrentMA == nil)				then EnhancedAssistOptions.CurrentMA = "-None-" end;
	if(EnhancedAssistOptions.CurrentSA == nil)				then EnhancedAssistOptions.CurrentSA = "-None-" end;
	if(EnhancedAssistOptions.Announce_Channels == nil)		then EnhancedAssistOptions.Announce_Channels = { ["RAID"] = 1 } end;
	if(EnhancedAssistOptions.Announce_Text == nil)			then EnhancedAssistOptions.Announce_Text = "%%assist $P on $T" end;
	if(EnhancedAssistOptions.Transparency == nil)			then EnhancedAssistOptions.Transparency = 0.75 end;
	if(EnhancedAssistOptions.AutoAssist == nil)				then EnhancedAssistOptions.AutoAssist = 0 end;
	if(EnhancedAssistOptions.UserMode == nil)				then EnhancedAssistOptions.UserMode = 1 end;
	if(EnhancedAssistOptions.HideOutOfGroup == nil)			then EnhancedAssistOptions.HideOutOfGroup = 0 end;
	if(EnhancedAssistOptions.SecondAssistOverride == nil)	then EnhancedAssistOptions.SecondAssistOverride = 1 end;
	if(EnhancedAssistOptions.Banner == nil)					then EnhancedAssistOptions.Banner = 1 end;
	if(EnhancedAssistOptions.Debug == nil)					then EnhancedAssistOptions.Debug = 0 end;
	if(EnhancedAssistOptions.Colors == nil)					then EnhancedAssistOptions.Colors = EA_DefaultColors end;
	if(EnhancedAssistOptions.Sounds == nil)					then EnhancedAssistOptions.Sounds = {} end;
	if(EnhancedAssistOptions.Sounds.Assist == nil)			then EnhancedAssistOptions.Sounds.Assist = "-None-" end;
	if(EnhancedAssistOptions.Sounds.Stop == nil)			then EnhancedAssistOptions.Sounds.Stop = "-None-" end;
	
	-- Old Version Mode Correction. Added in version 1.3.
	if(string.find(EnhancedAssistOptions.UserMode, "[a-z]"))	then EnhancedAssistOptions.UserMode = 1 end;
end

local function DisplayBool(setting)
	if ( setting == "1" or setting == 1 ) then
		return "On";
	elseif ( setting == "0" or setting == 0 ) then
		return "Off";
	end
	return setting;
end

local function print(message,Color)
	message = string.gsub(message, "%$(%w+)", EnhancedAssist_TokenReplacer)
	if ( Color ) then
		DEFAULT_CHAT_FRAME:AddMessage( message, Color.r, Color.g, Color.b);
	else
		DEFAULT_CHAT_FRAME:AddMessage( message, EnhancedAssistOptions.Colors.Message.r, EnhancedAssistOptions.Colors.Message.g, EnhancedAssistOptions.Colors.Message.b);
	end
end

local function printdebug(message)
	if ( EnhancedAssistOptions.Debug == 1 ) then
		DEFAULT_CHAT_FRAME:AddMessage( message, EnhancedAssistOptions.Colors.Debug.r, EnhancedAssistOptions.Colors.Debug.g, EnhancedAssistOptions.Colors.Debug.b);
	end
end

function PrintBannerMessage(message,Color)
	if ( not Color ) then
		Color = EnhancedAssistOptions.Colors.Banner;
	end
	if ( EnhancedAssistOptions.Banner == 1 ) then
		UIErrorsFrame:AddMessage(message, Color.r, Color.g, Color.b, 1.0, UIERRORS_HOLD_TIME);
	end
end

function EnhancedAssist_OnLoad()
	InitializeSetup();
	
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("CHAT_MSG_SAY");
	this:RegisterEvent("CHAT_MSG_PARTY");
	this:RegisterEvent("CHAT_MSG_RAID");
	this:RegisterEvent("CHAT_MSG_WHISPER");
	this:RegisterEvent("PARTY_MEMBERS_CHANGED");	-- For checking when the MAs live
	this:RegisterEvent("PLAYER_ENTERING_WORLD");	-- For checking when the MAs live
	this:RegisterEvent("PLAYER_LEAVING_WORLD");		-- To Unload crap when it doesn't need to be updated.
	-- this:RegisterEvent("PARTY_MEMBER_DISABLE");		-- For checking when the MAs live
	-- this:RegisterEvent("PARTY_MEMBER_ENABLE");		-- For checking when the MAs live
	-- this:RegisterEvent("RAID_ROSTER_UPDATE");		-- For checking when the MAs live
	this:RegisterEvent("UNIT_HEALTH");				-- For checking when the MAs live

	myClass = UnitClass("player"); -- ### Set just once instead of every freekin time the player does an assist
	myName = UnitName("player");
	
	EnhancedAssist_MA = 1;
	EnhancedAssist_SA = 2;
	
	EnhancedAssistOptions.CurrentMA = "";
	EnhancedAssistOptions.CurrentSA = "";

	SlashCmdList["ENHANCEDASSIST"] = EnhancedAssist_Command;

	SLASH_ENHANCEDASSIST1 = SLASH_ENHANCEDASSIST1;
	SLASH_ENHANCEDASSIST2 = SLASH_ENHANCEDASSIST2;
	SLASH_ENHANCEDASSIST3 = SLASH_ENHANCEDASSIST3;

	print( " " .. EA_CommandColor .. "Enhanced Assist " .. version .. " Loaded|r  /ea for advanced help", EnhancedAssistOptions.Colors.Help);
end

function EnhancedAssist_ResetToDefaults()
	EnhancedAssistOptions = {};
	InitializeSetup();
	EnhancedAssistOptionsFrame:Hide();
	if ( EnhancedAssistOptions.Show == 1 ) then
		EnhancedAssistFrame:Show();
		EnhancedAssistFrame_Background:SetAlpha(EnhancedAssistOptions.Transparency);
	else
		EnhancedAssistFrame:Hide();
		EnhancedAssistFrame_Background:SetAlpha(EnhancedAssistOptions.Transparency);
	end
	EnhancedAssist_updateCurrentAssists();
	print( "Settings restored to defaults", EnhancedAssistOptions.Colors.Help );
end

function EnhancedAssist_OnEvent()
	if ( event == "VARIABLES_LOADED" ) then
		this:UnregisterEvent(event);
		InitializeSetup();
		if ( EnhancedAssistOptions.Show == 1 ) then
			EnhancedAssistFrame:Show();
			EnhancedAssistFrame_Background:SetAlpha(EnhancedAssistOptions.Transparency);
		else
			EnhancedAssistFrame:Hide();
			EnhancedAssistFrame_Background:SetAlpha(EnhancedAssistOptions.Transparency);
		end
		-- EnhancedAssist_updateCurrentAssists();
		return;
	
	elseif ( event == "UNIT_HEALTH" ) then
		EnhancedAssist_PrepareAssistFrame(arg1);
		return;
	
	elseif ( event == "PARTY_MEMBERS_CHANGED" ) then
		printdebug("Running update from " .. event .. " event.");
		EnhancedAssist_PrepareAssistFrame();
		return;
	
	elseif ( event == "PLAYER_ENTERING_WORLD" ) then
		printdebug( myName .. " is reregistering and returning to the world.");
		this:RegisterEvent("PARTY_MEMBERS_CHANGED");
		this:RegisterEvent("UNIT_HEALTH");
		EnhancedAssist_PrepareAssistFrame(arg1);
		return;
	
	elseif ( event == "PLAYER_LEAVING_WORLD" ) then
		printdebug( myName .. " is unregistering and leaving world.");
		this:UnregisterEvent("PARTY_MEMBERS_CHANGED");
		this:UnregisterEvent("UNIT_HEALTH");
		return;
	--[[elseif (( event == "UNIT_HEALTH" ) or ( event == "PLAYER_ENTERING_WORLD" ) or ( event == "PARTY_MEMBERS_CHANGED" ) ) then
		AssistWindowHOOG();
		if (arg1) then
			if ( event ~= "UNIT_HEALTH" ) then
				printdebug("Running update from " .. event .. " event with arg1=" .. arg1 );
			end
			if ( UnitName(arg1) == EnhancedAssistOptions.CurrentMA ) then EnhancedAssist_updateSpecificAssist("CurrentMainAssist",EnhancedAssistOptions.CurrentMA,state); end
			if ( UnitName(arg1) == EnhancedAssistOptions.CurrentSA ) then EnhancedAssist_updateSpecificAssist("CurrentSecondAssist",EnhancedAssistOptions.CurrentSA,state); end
		else
			printdebug("Running update from " .. event .. " event." );
			EnhancedAssist_updateCurrentAssists();
		end]]
	-- elseif ( event == "UNIT_HEALTH" ) then
		-- local unitID = "";
		-- if arg1 then unitID = arg1 end
		-- printdebug(event .. " = " .. arg1 .. " name: " .. UnitName(arg1) );
		
		--  = GroupMemberIsSafe(UnitName(arg1),unitID);

	--[[elseif ( event == "CHAT_MSG_COMBAT_FRIENDLY_DEATH" ) then
		EnhancedAssist_updateCurrentAssists();
		local whoDied = string.gsub(arg1, " dies.", "");
		if ( whoDied == EnhancedAssistOptions.CurrentMA ) then
			getglobal(EnhancedAssistFrame:GetName() .. "CurrentMainAssist"):SetTextColor(1,0,0);
			PrintBannerMessage(whoDied .. " down!",1,0,0);
		elseif ( whoDied == EnhancedAssistOptions.CurrentSA ) then
			getglobal(EnhancedAssistFrame:GetName() .. "CurrentSecondAssist"):SetTextColor(1,0,0);
			PrintBannerMessage(whoDied .. " down!",1,0,0);
		elseif ( string.find(arg1, "dies") ) then 
			printdebug( whoDied .. " got pwnt and isn't the MA or SA.");
		end]]
	elseif (( event == "CHAT_MSG_SAY" ) or ( event == "CHAT_MSG_RAID" ) or ( event == "CHAT_MSG_PARTY" ) or ( event == "CHAT_MSG_WHISPER" )) then
		if ( arg1 ) then
			local i,j, cmd, param = string.find(arg1, "^ *([^ ]+) *(.*)$");
			if ( not cmd ) then cmd = ""; end
			cmd = string.lower(cmd);
 			-- local exiti,exitj = string.find(string.lower(arg1), "bonus honor");
			-- if (exitj) then printdebug("Exited because of misplaced event: " .. arg1); return 0; end
			-- if ( string.find(string.lower(arg1), "bonus honor") ) then return end
			-- arg2 = string.lower(arg2);
			
			if  ( param ) then
				if ( EnhancedAssistOptions.UserMode == 3 ) then
					if ( cmd == "%%setmh" ) then
						EnhancedAssist_setNewAssist( EnhancedAssist_MA, param );		
					elseif ( cmd == "%%setsh" ) then
						EnhancedAssist_setNewAssist( EnhancedAssist_SA, param );		
					end
				else
					if ( cmd == "%%setma" ) then
						EnhancedAssist_setNewAssist( EnhancedAssist_MA, param );		
					elseif ( cmd == "%%setsa" ) then
						EnhancedAssist_setNewAssist( EnhancedAssist_SA, param );		
					end
				end
			end
			-- 1 = DPS mode
			if ( EnhancedAssistOptions.UserMode == 1 ) then
				if ( EnhancedAssistOptions.AutoAssist == 1 ) then
					if ( (cmd == "%%assist" or cmd == "%%attack") and ( 
							string.lower(arg2) == string.lower(EnhancedAssistOptions.CurrentMA) or
							string.lower(arg2) == string.lower(EnhancedAssistOptions.CurrentSA)
						) ) then
						EnhancedAssist_doAssist(arg2,'Announce');
					end
				end
				if ( cmd == "%%stopdps" ) then
					-- SpellStopCasting();  -- Removed by blizzard from automation. needs button press.
					SpellStopTargeting();
					PrintBannerMessage( arg2 .. " has issued a Stop DPS command.", EnhancedAssistOptions.Colors.Message );
					if ( EA_Sounds[EnhancedAssistOptions.Sounds.Stop] ~= nil ) then
						PlaySoundFile(EA_Sounds[EnhancedAssistOptions.Sounds.Stop]);
					end
					if ( HasPetUI() ) then
						PetWait();
						PetStopAttack();
						PetFollow();
						printdebug( "Stopping Pet Attack." );
					end
					if ( UnitName("target") ) then
						ClearTarget();
						-- TargetLastTarget();
						printdebug( "Clearing Target and reselecting target." );
					end
				end
			-- 3 = Healer mode
			elseif ( EnhancedAssistOptions.UserMode == 3 ) then
				if ( EnhancedAssistOptions.AutoAssist == 1 ) then
					if ( cmd == "%%heal" and ( 
							string.lower(arg2) == string.lower(EnhancedAssistOptions.CurrentMA) or
							string.lower(arg2) == string.lower(EnhancedAssistOptions.CurrentSA)
						) ) then
						EnhancedAssist_doAssist(arg2,'Announce');
					end
				end
			end
			--[[if ( string.lower(cmd) == "%%follow" ) then
				FollowByName(arg2);
			end]]
		end
	--[[elseif ( event == "RAID_ROSTER_UPDATE" ) then
		local note = "";
		if arg1 then note = arg1 end
		printdebug(event .. " = " .. note);
		EnhancedAssist_updateCurrentAssists();]]
		return;
	end
end

function EnhancedAssist_PrepareAssistFrame(eventSender)
	AssistWindowHOOG();
	if (eventSender) then
		--[[if ( event ~= "UNIT_HEALTH" ) then
			printdebug("Running update from " .. event .. " event with arg1=" .. arg1 );
		end]]
		if ( UnitName(eventSender) == EnhancedAssistOptions.CurrentMA ) then EnhancedAssist_updateSpecificAssist("_CurrentMA",EnhancedAssistOptions.CurrentMA,state); end
		if ( UnitName(eventSender) == EnhancedAssistOptions.CurrentSA ) then EnhancedAssist_updateSpecificAssist("_CurrentSA",EnhancedAssistOptions.CurrentSA,state); end
	else
		-- printdebug("Running update from " .. event .. " event." );
		EnhancedAssist_updateCurrentAssists();
	end
end

function EnhancedAssist_doAssist(theAssist,announceToo)
	local abortAssist = 0;
	if ( theAssist == UnitName("player") ) then
		-- ### You *are* the assist, don't be dumb and assist yourself noob.
		printdebug( "You *are* the assist, don't be dumb and assist yourself noob." );
		abortAssist = 1;
	elseif ( theAssist == nil or theAssist == "" ) then
		-- ### GG coo-guy, there is no assist
		printdebug( "GG coo-guy, there is no assist..." );
		abortAssist = 1;
	elseif ( EnhancedAssistOptions.SecondAssistOverride == 0 and theAssist ~= EnhancedAssistOptions.CurrentMA ) then
		-- ### Secondary assister is not allowed to override the assist call of the MA, and the MA wasn't the one who called the assist.
		printdebug( theAssist .. " is not allowed to override the assist call of the MA, and the MA wasn't the one who called the assist." );
		abortAssist = 1;
	elseif ( LootFrame:IsVisible() ) then
		printdebug( "You are are currently looting " .. UnitName("target") .. "." );
		abortAssist = 1;
	else
		local lastTargetName = UnitName("target");
		-- ### Set assist to target for checking stuff
		TargetByName( theAssist );
		if ( UnitName("target") == nil or UnitName("target") ~= theAssist ) then
			print( theAssist .. " is not targetable for assisting.", EnhancedAssistOptions.Colors.Error );
			abortAssist = 1;
--[[		elseif ( UnitName("targettarget") == nil ) then
			print( theAssist .. " has no target.", EnhancedAssistOptions.Colors.Error );
			abortAssist = 1;
		elseif ( UnitName("targettarget") == theAssist ) then
			printdebug( theAssist .. " is not targeting anything.", EnhancedAssistOptions.Colors.Error );
			abortAssist = 1;]]
		elseif ( UnitIsGhost("target") ) then
			print( theAssist .. " has already released spirit.", EnhancedAssistOptions.Colors.Error );
			abortAssist = 1;
		end
		-- ### Reset target to whatever the player had before this section started.
		if ( lastTargetName ) then
			TargetLastTarget();
		else
			ClearTarget();
		end
		-- printdebug( "GG coo-guy, there is no assist..." );
	end

	if ( abortAssist == 0 ) then
		ClearTarget();
		AssistThis( theAssist );
		if ( UnitName("target") and announceToo ) then
			PrintBannerMessage("Assisting " .. theAssist .. " on " .. UnitName("target") .. ".");
			-- PlaySound("igMainMenuContinue");
			if ( EA_Sounds[EnhancedAssistOptions.Sounds.Assist] ~= nil ) then
				PlaySoundFile(EA_Sounds[EnhancedAssistOptions.Sounds.Assist]);
			end
		end
	end
end

function AssistWindowHOOG()
	if ( EnhancedAssistOptions.Show == 1 ) then
		if ( EnhancedAssistOptions.HideOutOfGroup == 1 and not GetPartyMember(1) ) then
			EnhancedAssistFrame:Hide();
		else
			EnhancedAssistFrame:Show();
			EnhancedAssistFrame_Background:SetAlpha(EnhancedAssistOptions.Transparency);
		end
	end
end
-- GetPartyMember(id) 
function GetUnitIdByName(checkingFor)
	local loopCount = 0;
	local groupString = "";
	if ( UnitName("player") == checkingFor ) then
		printdebug( "Unit is the player." );
		return "player";
	elseif ( UnitInRaid("player") ) then
		-- Check all the raid members until the person is found, then break/return
		-- printdebug( "Player is in raid. checking raid members." );
		for i = 1, MAX_RAID_MEMBERS do
			if ( UnitName("raid" .. i) == checkingFor ) then
				return "raid" .. i;
			end
		end
		printdebug( checkingFor .. " was not a raid member. checking pets now." );
		-- Check for pets if name isn't a party member
		for i = 1, MAX_RAID_MEMBERS do
			if ( UnitName("raidpet" .. i) == checkingFor ) then
				return "raidpet" .. i;
			end
		end
		printdebug( checkingFor .. " was not found in raid." );
	elseif ( UnitInParty("party1") ) then
		-- Check all the party members until the person is found, then break/return
		-- printdebug( "Player is in Party. checking party members." );
		for i = 1, 4 do
			if ( UnitName("party" .. i) == checkingFor ) then
				return "party" .. i;
			end
		end
		printdebug( checkingFor .. " was not a party member. checking pets now." );
		-- Check for pets if name isn't a party member
		for i = 1, 4 do
			if ( UnitName("partypet" .. i) == checkingFor ) then
				return "partypet" .. i;
			end
		end
		printdebug( checkingFor .. " was not found in party." );
	end
	if ( UnitName("pet") == checkingFor ) then
		printdebug( "Unit is the player's pet." );
		return "pet";
	end
	-- if ( loopCount > 0 ) then
	-- end
				--[[if ( not online ) then
					printdebug( checkingFor .. " is offline" );
					return "offline";
				elseif ( isDead  ) then
					printdebug( checkingFor .. " is dead" );
					return "dead";
				end]]
	printdebug( "Crap, I give up. I can't find " .. checkingFor .. " anywhere." );
	return nil;
end

function AssistThis( theAssist )
	local unitID = GetUnitIdByName( theAssist );
	if ( unitID ) then
		printdebug( "Initiating assist of " .. unitID .. " named " .. theAssist );
		AssistUnit( unitID );
	else
		printdebug( "Initiating assist by name because " .. theAssist .. " couldn't be located.");
		AssistByName( theAssist );
	end
	if ( GetCVar("assistAttack") == "1" ) then
		if ( myClass == "Hunter" ) then
			-- ### Because of the anti-botting code blizz added (for good reason) the following
			-- ### code will only activate upon a button-press event, like the assist keybind.
			CastSpell( GetSpellByName( "Auto Shot" ) );
		end
	end
end

function GetSpellByName(requestedSpellName)
	local id,tab;
	for i = 1, GetNumSpellTabs(), 1 do
		local name, texture, offset, numSpells = GetSpellTabInfo(i);
		for y = 1, numSpells, 1 do
			local spellName, rankName = GetSpellName(offset+y, BOOKTYPE_SPELL);
			if ( string.find( spellName,requestedSpellName ) ) then
				id = y+offset;
				tab = i;
			-- else
				-- print( 'skipping ' .. spellName .. ' id='.. i ..'.');
			end
		end
	end
	return id,tab;
end

function EnhancedAssist_ToggleDisplay(param)
	local windowState = EnhancedAssist_Toggle("Show",param);
	if (windowState == 0 ) then
		EnhancedAssistFrame:Hide();
	elseif ( windowState == 1 ) then
		EnhancedAssistFrame:Show();
		EnhancedAssist_updateCurrentAssists();
	end
end

function EnhancedAssist_updateCurrentAssists()
	EnhancedAssist_updateSpecificAssist("_CurrentMA",EnhancedAssistOptions.CurrentMA);
	EnhancedAssist_updateSpecificAssist("_CurrentSA",EnhancedAssistOptions.CurrentSA);
	-- When the user mode is set to healer update the labels of the MA and SA to the MH and MH labels
	if (EnhancedAssistOptions.UserMode == 3) then
		EnhancedAssistFrame_CurrentMALabel_Text:SetText(ENHANCEDASSISTFRAME_CURRENTMHLABEL);
		EnhancedAssistFrame_CurrentSALabel_Text:SetText(ENHANCEDASSISTFRAME_CURRENTSHLABEL);
	else
		EnhancedAssistFrame_CurrentMALabel_Text:SetText(ENHANCEDASSISTFRAME_CURRENTMALABEL);
		EnhancedAssistFrame_CurrentSALabel_Text:SetText(ENHANCEDASSISTFRAME_CURRENTSALABEL);
	end
end

function EnhancedAssist_updateSpecificAssist(whichAssist, assistName, unitID)
	if ( not unitID ) then
		unitID = GetUnitIdByName( assistName );
	end
	-- if ( GetPartyMember(1) and EnhancedAssistOptions.HideOutOfGroup ) then
		if ( assistName ~= nil or assistName ~= "" ) then
			-- printdebug( "EnhancedAssistFrame" .. whichAssist .. "Name : " .. getglobal("EnhancedAssistFrame" .. whichAssist .. "Name"):GetText() );
			getglobal("EnhancedAssistFrame" .. whichAssist .. "Name_Text"):SetText(assistName);
			local newWidth = 0;
			if ( EnhancedAssistFrame_CurrentMAName_Text:GetWidth() > EnhancedAssistFrame_CurrentSAName_Text:GetWidth() ) then
				newWidth = EnhancedAssistFrame_CurrentMAName_Text:GetWidth();
			else
				newWidth = EnhancedAssistFrame_CurrentSAName_Text:GetWidth();
			end
			EnhancedAssistFrame_CurrentMAName:SetWidth( newWidth );
			EnhancedAssistFrame_CurrentSAName:SetWidth( newWidth );
			newWidth = 42 + newWidth + 14;
			EnhancedAssistFrame_Background:SetWidth(newWidth);
			EnhancedAssistFrame:SetWidth(newWidth);
			
			if ( unitID ) then
				if ( not UnitIsConnected(unitID) ) then
					getglobal("EnhancedAssistFrame" .. whichAssist .. "Name_Text"):SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
					printdebug( whichAssist .. ":" .. assistName .. " is set to offline." );
				elseif ( UnitIsDeadOrGhost(unitID) ) then
					getglobal("EnhancedAssistFrame" .. whichAssist .. "Name_Text"):SetTextColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
					printdebug( whichAssist .. ":" .. assistName .. " is set to dead." );
				else
					getglobal("EnhancedAssistFrame" .. whichAssist .. "Name_Text"):SetTextColor(1,1,1);
					-- printdebug( whichAssist .. ":" .. assistName .. " is set to safe." );
				end
			else
				getglobal("EnhancedAssistFrame" .. whichAssist .. "Name_Text"):SetTextColor(0.5,1,1);
				printdebug( assistName .. " is not in raid." );
			end
		else
			getglobal("EnhancedAssistFrame" .. whichAssist .. "Name_Text"):SetTextColor(1,0,0);
			getglobal("EnhancedAssistFrame" .. whichAssist .. "Name_Text"):SetText(ENHANCEDASSIST_NO_ASSIST_SET);
			printdebug( whichAssist .. " is undefined." );
		end
	-- end
end

function EnhancedAssist_setNewAssist_WithAnn(assistType, newAssist)
	local targetName = UnitName("target");
	local theAssist;
	if ( newAssist ) then theAssist = newAssist;
	elseif ( UnitName("target") ) then theAssist = UnitName("target");
	end
	
	-- local oldAssist = 0; 
	-- if (  ) then
	-- 	oldAssist = 1;
	-- end
	if ( theAssist ) then
		if ( IsRaidLeader() or IsRaidOfficer() or IsPartyLeader() -- or 
			-- UnitName("player") == EnhancedAssistOptions.CurrentMA or 
			-- UnitName("player") == EnhancedAssistOptions.CurrentSA
			) then
			if		( assistType == EnhancedAssist_MA ) then
				EA_ChatMessage( "%%setma " .. theAssist );
			elseif	( assistType == EnhancedAssist_SA ) then
				EA_ChatMessage( "%%setsa " .. theAssist );
			end
		end
		EnhancedAssist_setNewAssist(assistType, newAssist);
	end
end

function EnhancedAssist_setNewAssist(assistType, newAssist)
	local targetName = UnitName("target");
	local theAssist;
	if ( newAssist ) then theAssist = newAssist;
	elseif ( UnitName("target") ) then theAssist = UnitName("target");
	end

	if ( theAssist ) then
		theAssist = ucfirst(theAssist);
		if ( UnitCanAssist("player", "target") and UnitIsVisible("target") ) then
			print( theAssist .. " verified as valid assist.");
		end		-- #### Add the check to make sure the target is assistable
		-- local raText = "";
		-- This is used for the check down in the announcment below
		if ( assistType == EnhancedAssist_MA ) then
			EnhancedAssistOptions.CurrentMA = theAssist;
			print( ENHANCEDASSIST_SET_MA .. theAssist);
		elseif ( assistType == EnhancedAssist_SA ) then
			EnhancedAssistOptions.CurrentSA = theAssist;
			print( ENHANCEDASSIST_SET_SA .. theAssist);
		end
		-- if ( IsRaidLeader() or IsRaidOfficer() or IsPartyLeader() or oldAssist ) then
			-- EA_ChatMessage(raText);
			--[[for channel,index in EnhancedAssistOptions.Announce_Channels do
				SendChatMessage(string.gsub(raText, "%$(%w+)", EnhancedAssist_TokenReplacer), channel);
				printdebug( "Channel: " .. channel .. " at index: " .. index );
			end]]
		-- end
		printdebug( "Setting " .. theAssist .. " as " .. assistType );
	else
		print( "Unable to set assist to that unit.", EnhancedAssistOptions.Colors.Error );
	end

	EnhancedAssist_updateCurrentAssists();	
end

function EnhancedAssist_CallAssist(toWhom)
	if ( UnitName("target") ) then
		EA_ChatMessage(EnhancedAssistOptions.Announce_Text,toWhom);
		--[[if ( toWhom ) then
			SendChatMessage(string.gsub(EnhancedAssistOptions.Announce_Text, "%$(%w+)", EnhancedAssist_TokenReplacer), "WHISPER", this.language, toWhom);
		else
			for channel,index in EnhancedAssistOptions.Announce_Channels do
				SendChatMessage(string.gsub(EnhancedAssistOptions.Announce_Text, "%$(%w+)", EnhancedAssist_TokenReplacer), channel);
			end
		end]]
	else
		print( "Please target something before calling assist.", EnhancedAssistOptions.Colors.Error );			
	end
end

function EnhancedAssist_StopDPS(toWhom)
	local onTarget = "";
	if ( UnitName("target") ) then
		onTarget = " on $T";
	end
	EA_ChatMessage("%%stopdps" .. onTarget,toWhom)
	--[[if ( toWhom ) then
		SendChatMessage(string.gsub("%%stopdps" .. onTarget , "%$(%w+)", EnhancedAssist_TokenReplacer), "WHISPER", this.language, toWhom);
	else
		for channel,index in EnhancedAssistOptions.Announce_Channels do
			SendChatMessage(string.gsub("%%stopdps" .. onTarget , "%$(%w+)", EnhancedAssist_TokenReplacer), channel);
		end
	end]]
end

function EnhancedAssist_Toggle(setting,param)
	if ( string.lower(param) == "off" or param == 0 ) then
		EnhancedAssistOptions[setting] = 0;
	elseif ( string.lower(param) == "on" or param == 1 ) then
		EnhancedAssistOptions[setting] = 1;
	else
		if (EnhancedAssistOptions[setting] == 1 ) then
			EnhancedAssistOptions[setting] = 0;
		else
			EnhancedAssistOptions[setting] = 1;
		end
	end
	return EnhancedAssistOptions[setting];
end

function EnhancedAssist_AddSoundPack(EA_SoundPack)
	for title,file in EA_SoundPack do
		EA_Sounds[title] = file;
	end
end

function EnhancedAssist_TokenReplacer(token)
	if( token == "MA" ) then
		return EnhancedAssistOptions.CurrentMA;
	elseif (token == "SA" ) then
		return EnhancedAssistOptions.CurrentSA;
	elseif (token == "T" ) then
		return UnitName("target");
	elseif (token == "P" ) then
		return UnitName("player");
	else
		return token;
	end
end

function ucfirst(str)
	return string.upper(string.sub(str,1,1)) .. string.sub(str,2);
end

function EA_ChatMessage(message,toWhom) -- To Whom is an optional whisper target
	if ( toWhom ~= '' and toWhom ~= nil ) then
		SendChatMessage(string.gsub(message, "%$(%w+)", EnhancedAssist_TokenReplacer), "WHISPER", this.language, toWhom);
	else
		for channel,index in EnhancedAssistOptions.Announce_Channels do
			SendChatMessage(string.gsub(message, "%$(%w+)", EnhancedAssist_TokenReplacer), channel);
			printdebug( "say to " .. channel .. " this: " .. message );
		end
	end
end

function EnhancedAssist_Command(command)
	command = string.lower(command);
	local i,j, cmd, param = string.find(command, "^([^ ]+) +(.+) *$");
	if (not cmd) then cmd = command; end
	if (not cmd) then cmd = ""; end
	if (not param) then param = ""; end

	if ( (cmd == nil) or (cmd == "") or (cmd == "help") ) then
		print( "Enhanced Assist v" .. version .. "  - By Gearshaft -", EnhancedAssistOptions.Colors.Help );
		print( "\nCommands:\n", DefaultColor);
		local mainAssist = ENHANCEDASSIST_NO_ASSIST_SET;
		local secondAssist = ENHANCEDASSIST_NO_ASSIST_SET;
		if ( (EnhancedAssistOptions.CurrentMA ~= nil) and (EnhancedAssistOptions.CurrentMA ~= "") ) then
			mainAssist = EnhancedAssistOptions.CurrentMA;
		end
		if ( (EnhancedAssistOptions.CurrentSA ~= nil) and (EnhancedAssistOptions.CurrentSA ~= "") ) then
			secondAssist = EnhancedAssistOptions.CurrentSA;
		end
		print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " ui|r  Bring up the options window.", EnhancedAssistOptions.Colors.Help );
		print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " [show \| hide]|r " .. EA_SettingColor .. "[" .. DisplayBool( EnhancedAssistOptions.Show ) .. "]|r  Show or hide the Assisting window.", EnhancedAssistOptions.Colors.Help );
		print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " ma <name>|r " .. EA_SettingColor .. "[" .. mainAssist .. "]|r  Set main assist", EnhancedAssistOptions.Colors.Help );
		print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " sa <name>|r " .. EA_SettingColor .. "[" .. secondAssist .. "]|r  Set second assist", EnhancedAssistOptions.Colors.Help );
		print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " mode [dps \| tank \| healer]|r " .. EA_SettingColor .. "[" .. EA_Modes[EnhancedAssistOptions.UserMode] .. "]|r  Sets the user mode. Set this based on your role in the group.", EnhancedAssistOptions.Colors.Help );
		print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " callassist {WhisperToName}|r  Announce to the specified channels that others should assist you. Optional: Add a player name after a space to whisper the message just to that player.", EnhancedAssistOptions.Colors.Help );
		print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " stop/stopdps|r  Announce to the specified channels tor everyone to stop DPS on the assist target.", EnhancedAssistOptions.Colors.Help );
		print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " assistmain|r  Assist the main assist.", EnhancedAssistOptions.Colors.Help );
		print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " assistsecond|r  Assist the secondary assist.", EnhancedAssistOptions.Colors.Help );
		print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " anntext <string>|r " .. EA_SettingColor .. "[" .. EnhancedAssistOptions.Announce_Text .. "]|r  Text to announce main assist", EnhancedAssistOptions.Colors.Help );
		local annChannels = "";
		for channel, index in EnhancedAssistOptions.Announce_Channels do
			if (index > 1) then
				annChannels = annChannels .. ", " .. channel; 
			else
				annChannels = channel; 
			end
		end
		print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " annch [SAY \| PARTY \| RAID]|r " .. EA_SettingColor .. "[" .. annChannels .. "]|r  Channel to announce assist calls assist to. Can be set to one or multiple channels.", EnhancedAssistOptions.Colors.Help );
		print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " trans (0.0-1.0)|r " .. EA_SettingColor .. "[" .. EnhancedAssistOptions.Transparency .. "]|r  Amount of assist window transparency (0=invis, 1=fully vis)", EnhancedAssistOptions.Colors.Help );
		print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " autoassist [Off\|On] or blank to toggle|r " .. EA_SettingColor .. "[" .. DisplayBool( EnhancedAssistOptions.AutoAssist ) .. "]|r  Enable/Disable automatic assisting when called.", EnhancedAssistOptions.Colors.Help );
		print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " banner [Off\|On] or blank to toggle|r " .. EA_SettingColor .. "[" .. DisplayBool( EnhancedAssistOptions.Banner ) .. "]|r  Sets wether or not a banner message will print to the middle of your screen on assist target change.", EnhancedAssistOptions.Colors.Help );
		print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " hideoutofgroup [Off\|On] or blank to toggle|r " .. EA_SettingColor .. "[" .. DisplayBool( EnhancedAssistOptions.HideOutOfGroup ) .. "]|r  This will hide the Assisting window when you are not in a group.", EnhancedAssistOptions.Colors.Help );
	elseif ( cmd == "hide" ) then
		print( "Hiding assist window.", EnhancedAssistOptions.Colors.Help );
		EnhancedAssist_ToggleDisplay(0);
	elseif ( cmd == "show" ) then
		print( "Displaying assist window.", EnhancedAssistOptions.Colors.Help );
		EnhancedAssist_ToggleDisplay(1);
	elseif ( cmd == "ui" ) then
		if (EnhancedAssistOptionsFrame:IsVisible()) then
			HideUIPanel(EnhancedAssistOptionsFrame);
		else
			ShowUIPanel(EnhancedAssistOptionsFrame);
			-- AutoEmoteQuestFrame:Show();
		end
	elseif ( cmd == "reset" or cmd == "default" or cmd == "defaults" ) then
		EnhancedAssist_ResetToDefaults();
	elseif ( cmd == "ma" ) then
		EnhancedAssist_setNewAssist( EnhancedAssist_MA, param );
	elseif ( cmd == "sa" ) then
		EnhancedAssist_setNewAssist( EnhancedAssist_SA, param );
	elseif ( cmd == "mode" ) then
		if ( param == "dps" ) then
			EnhancedAssistOptions.UserMode = 1;
			print( ENHANCEDASSIST_MODE_SETTING .. EA_Modes[1] );
		elseif ( param == "tank" ) then
			EnhancedAssistOptions.UserMode = 2;
			print( ENHANCEDASSIST_MODE_SETTING .. EA_Modes[2] );
		elseif ( param == "healer" ) then
			EnhancedAssistOptions.UserMode = 3;
			print( ENHANCEDASSIST_MODE_SETTING .. EA_Modes[3] );
		else
			print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " mode|r must be [dps \| tank \| healer]", EnhancedAssistOptions.Colors.Help );
		end
	elseif ( cmd == "callassist" ) then
		if ( param ~= "" ) then
			EnhancedAssist_CallAssist(param);
		else
			EnhancedAssist_CallAssist();
		end
	elseif ( cmd == "stop" or cmd == "stopdps" ) then
		if ( param ) then
			EnhancedAssist_StopDPS(param);
		else
			EnhancedAssist_StopDPS();
		end
	elseif ( cmd == "anntext" ) then
		if( param ~= "" ) then
			EnhancedAssistOptions.Announce_Text = param;
			print( "Announce text for callassist set to: " .. param, EnhancedAssistOptions.Colors.Help );
		else
			print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " anntext|r must not be blank", EnhancedAssistOptions.Colors.Help );
		end
	elseif ( cmd == "annch" ) then
		param = string.upper(param);
		local chCount = 0;
		local channelsSet = "";
		local tempChannelSaverForNoobs = EnhancedAssistOptions.Announce_Channels;
		EnhancedAssistOptions.Announce_Channels = {};
		for ch in string.gfind(param, "%a+") do
			if( ch == "SAY" or ch == "PARTY" or ch == "RAID" ) then
				chCount = chCount + 1;
				EnhancedAssistOptions.Announce_Channels[ch] = chCount;
				if ( chCount == 1 ) then
					channelsSet = ch;
				else
					channelsSet = channelsSet .. "," .. ch;
				end
			end
		end
		if ( chCount > 0 ) then
			print( "Announce channel for callassist set to: {" .. channelsSet .. "}", EnhancedAssistOptions.Colors.Help );
		else
			EnhancedAssistOptions.Announce_Channels = tempChannelSaverForNoobs;
			print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " annch|r must be one or more of the following: SAY, PARTY, or RAID", EnhancedAssistOptions.Colors.Help );
		end
	elseif ( cmd == "trans" ) then
		if( (param ~= "") and ((param-0) >= 0) and ((param-0) <= 1)) then
			EnhancedAssistOptions.Transparency = (param-0);
			EnhancedAssistFrameBackground:SetAlpha( param );
			print( ENHANCEDASSIST_TRANSPARENCY_SET .. EnhancedAssistOptions.Transparency);
		else
			print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " trans (0.0-1.0)|r " .. EA_SettingColor .. "["..EnhancedAssistOptions.Transparency.."]|r  Amount of assist window transparency (0=invis, 1=fully vis)", EnhancedAssistOptions.Colors.Help );
		end
	elseif ( cmd == "assistmain" ) then
		EnhancedAssist_doAssist(EnhancedAssistOptions.CurrentMA);
	elseif ( cmd == "assistsecond" ) then		
		EnhancedAssist_doAssist(EnhancedAssistOptions.CurrentSA);
	elseif ( cmd == "autoassist" ) then
		if ( param == "off" or param == "on" or param == 1 or param == 0 or param == "" or param == nil ) then
			EnhancedAssist_Toggle("AutoAssist",param);
			print( ENHANCEDASSIST_AUTOASSIST_CALLS .. DisplayBool( EnhancedAssistOptions.AutoAssist ) );
		else
			print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " autoassist|r must be [Off\|On] or empty to toggle.", EnhancedAssistOptions.Colors.Help );
		end
		-- EnhancedAssistOptions.Debug
	elseif ( cmd == "banner" ) then
		if ( param == "off" or param == "on" or param == 1 or param == 0  or param == "" or param == nil ) then
			EnhancedAssist_Toggle("Banner",param);
			print( ENHANCEDASSIST_BANNER_CALLS .. DisplayBool( EnhancedAssistOptions.Banner ) );
		else
			print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " banner|r must be [Off\|On] or empty to toggle.", EnhancedAssistOptions.Colors.Help );
		end
	elseif ( cmd == "hideoutofgroup" ) then
		if ( param == "off" or param == "on" or param == 1 or param == 0  or param == "" or param == nil ) then
			EnhancedAssist_Toggle("HideOutOfGroup",param);
			print( ENHANCEDASSIST_HIDEOOG .. DisplayBool( EnhancedAssistOptions.HideOutOfGroup ) );
		else
			print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " hideoutofgroup|r must be [Off\|On] or empty to toggle.", EnhancedAssistOptions.Colors.Help );
		end
		AssistWindowHOOG();
	elseif ( cmd == "debug" ) then
		if ( param == "off" or param == "on" or param == 1 or param == 0 or param == "" or param == nil ) then
			EnhancedAssist_Toggle("Debug",param);
			print( ENHANCEDASSIST_DEBUG_MESSAGES .. DisplayBool( EnhancedAssistOptions.Debug ) );
		else
			print( " " .. EA_CommandColor .. SLASH_ENHANCEDASSIST1 .. " debug|r must be [Off\|On] or empty to toggle.", EnhancedAssistOptions.Colors.Help );
		end
	end
	-- Update the window so if it is open the changes take effect right away.
	EnhancedAssistOptionsFrame_OnShow();
	EnhancedAssist_updateCurrentAssists();
end
